/*
    Copyright 2004,2005,2006,2007,2008,2009 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>

#ifdef WIN32
    #include <windows.h>
    #include <wincrypt.h>
#else
    #define stricmp strcasecmp
    #define stristr strcasestr
#endif

typedef unsigned char   u8;



#define VER         "0.2"



int trillian_pwd(u8 *pwd, int old_mode);
u8 *base64_decode(u8 *data, int *size);
void std_err(void);
void wait_exit(int ret);



int main(int argc, char *argv[]) {
    FILE    *fd;
    int     old_mode    = 0;
    u8      buff[1024],
            *fname,
            *p;

    fputs("\n"
        "Trillian Astra password decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <file.INI/password> [old_mode]\n"
            "\n"
            "The INI files containing the passwords (usually profiles.ini, aim.ini, msn.ini\n"
            "and yahoo.ini) can be located in 2 folders:\n"
            "- into the Trillian users'folder\n"
            "- into the Documents and Setting folder\n"
            "\n", argv[0]);
        wait_exit(1);
    }
    fname = argv[1];
    if(argc > 2) old_mode = 1;

    fd = fopen(fname, "rb");
    if(!fd) {
        p = strdup(fname);  // not needed
        trillian_pwd(p, old_mode);
        printf("\n  PASSWORD: %s\n", p);
    } else {
        while(fgets(buff, sizeof(buff), fd)) {
            for(p = buff; *p && (*p != '\r') && (*p != '\n'); p++);
            *p = 0;

            p = strchr(buff, '=');
            if(!p) continue;
            *p++ = 0;
            if(stristr(buff, "password") && !stristr(buff, "save")) {
                trillian_pwd(p, old_mode);
                printf("  %-10s %s\n", buff, p);
            } else if(stristr(buff, "account")) {
                printf("\n  %-10s %s\n", buff, p);
            }
        }
        fclose(fd);
    }

    wait_exit(0);
    return(0);
}



int trillian_pwd(u8 *ret_pwd, int old_mode) {
    int             t,
                    pwdlen;
    unsigned char   key[] =
                    "\xF3\x26\x81\xC4\x39\x86\xDB\x92\x71\xA3\xB9\xE6\x53\x7A\x95\x7C",
                    *pwd,
                    *p,
                    *o;

    if(old_mode) {
        pwd    = ret_pwd;
        pwdlen = strlen(pwd);
    } else {
        pwdlen = -1;
        pwd = base64_decode(ret_pwd, &pwdlen);
    }

    p = key;
    for(o = ret_pwd; sscanf(pwd, "%02X", &t) == 1; o++) {
        *o = t ^ *p;
        pwd += 2;
        p++;
        if(!*p) p = key;
    }
    *o = 0;
    return(0);
}



u8 *base64_decode(u8 *data, int *size) {
    int     len,
            xlen,
            a   = 0,
            b   = 0,
            c,
            step;
    u8      *buff,
            *limit,
            *p;
    static const u8 base[128] = {   // supports also the Gamespy base64
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3e,0x00,0x00,0x00,0x3f,
        0x34,0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,0x3c,0x3d,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,
        0x0f,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x3e,0x00,0x3f,0x00,0x00,
        0x00,0x1a,0x1b,0x1c,0x1d,0x1e,0x1f,0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,
        0x29,0x2a,0x2b,0x2c,0x2d,0x2e,0x2f,0x30,0x31,0x32,0x33,0x00,0x00,0x00,0x00,0x00
    };

    if(!size || (*size < 0)) {      // use size -1 for auto size!
        len = strlen(data);
    } else {
        len = *size;
    }
    xlen = ((len >> 2) * 3) + 1;    // NULL included in output for text
    buff = malloc(xlen);
    if(!buff) return(NULL);

    p = buff;
    limit = data + len;

    for(step = 0; /* data < limit */; step++) {
        do {
            if(data >= limit) {
                c = 0;
                break;
            }
            c = *data;
            data++;
            if((c == '=') || (c == '_')) {  // supports also the Gamespy base64
                c = 0;
                break;
            }
        } while(c && ((c <= ' ') || (c > 0x7f)));
        if(!c) break;

        switch(step & 3) {
            case 0: {
                a = base[c];
                break;
            }
            case 1: {
                b = base[c];
                *p++ = (a << 2)        | (b >> 4);
                break;
            }
            case 2: {
                a = base[c];
                *p++ = ((b & 15) << 4) | (a >> 2);
                break;
            }
            case 3: {
                *p++ = ((a & 3) << 6)  | base[c];
                break;
            }
        }
    }
    *p = 0;

    len = p - buff;
    if(size) *size = len;
    if((len + 1) != xlen) {             // save memory
        buff = realloc(buff, len + 1);  // NULL included!
        if(!buff) return(NULL);
    }
    return(buff);
}



void std_err(void) {
    perror("\nError");
    wait_exit(1);
}



void wait_exit(int ret) {
    fprintf(stderr, "\npress RETURN to exit\n");
    fgetc(stdin);
    exit(ret);
}


